## SQL script to alter the Rscript table


# Inserting new records in data_format table using procedures. This done only once when new data formats are added, so each procedure is
# used only once. 
# Sourcing:
# SOURCE /var/www/Public/CTFSRPackage/system/application/libraries/Rscript.sql;
# SOURCE /var/www/html/Public/CTFSRPackage/system/application/libraries/Rscript.sql;
# SOURCE /home/bradleyguest/public_html/Tobias/CTFSRPackage/system/application/libraries/Rscript.sql;
# Executing. Note this is only done once then never again unless entire database is recreated. June 2011
# CALL AddSpeciesColumns(1);
# CALL AddWSGColumns(1);
# CALL AddElevColumns(1);


DELIMITER //

DROP PROCEDURE IF EXISTS AddSpeciesColumns //
CREATE PROCEDURE AddSpeciesColumns(IN addcol INT)
BEGIN

	IF addcol = 1 
		THEN ALTER TABLE data_format ADD COLUMN is_spp_data TINYINT DEFAULT 0;
	END IF;
	
	UPDATE data_format SET is_spp_data=1 WHERE column_name='sp';
	UPDATE data_format SET description=
	 CONCAT('The species mnemonic. Full Latin name can be found in the R Analytical Species Table (spptable) for each plot, or in the ',
	        'Taxonomy Report at http:/ctfs.arnarb.harvard.edu/CTFSReports. This mnemonic is crucial in joining various databases, for instance ',
	        'The Full Tree and Stem tables to the Species tables and Wood Density Table') 
	WHERE column_name='sp';
	
	INSERT INTO data_format (column_name,description,is_spp_data) VALUES
	 ('Latin','Full Genus + Species',1),
	 ('Family','Family according to APG',1),
	 ('speciesID','Unique species identifier in MySQL database; seldom needed',1),
	 ('authority','Taxanomic authority for species name',1),
	 ('IDlevel','Level of identification: can be species, genus, or family; none means no identification at all; multi indicates that a code includes a mix of more than one species',1),
	 ('syn','Synonyms formerly used in the plot',1),
	 ('subsp','Subspecies name',1);

END //


DROP PROCEDURE IF EXISTS AddWSGColumns //
CREATE PROCEDURE AddWSGColumns(IN addcol INT)
BEGIN

	IF addcol = 1 THEN
		ALTER TABLE data_format ADD COLUMN is_wsg_data TINYINT DEFAULT 0;
	END IF;

	UPDATE data_format SET is_wsg_data=1 WHERE column_name='sp';	
	INSERT INTO data_format (column_name,description,is_wsg_data) VALUES
     ('wsg','Wood specific gravity (dry wood density)',1),
     ('idlevel',
        CONCAT('The taxonomic level from which the WSG was taken. If genus, wsg is the mean for the entire genus, and likewise for family. ',
               'This is not the same as IDlevel in the species table.'),
       1),
     ('site','Plot name, matching the site name in the main database',1),
     ('genus','Genus name',1),
     ('species','Species name; note that a Genus-species name will appear in this table twice (or more), in different plots with different sp codes',1),
     ('genwood','Mean WSG for the genus; if idlevel=genus, this is the value in the wsg column',1),
     ('famwood','Mean WSG for the family; if idlevel=family, this is the value in the wsg column',1),
     ('spwood','Mean WSG for the species; iflevel=species, this is the value in the wsg column',1),
     ('count','Internal variable, used in creating table and not in analyses',1),
     ('groupID','Internal variable, used in creating table and not in analyses',1);
     
END //


DROP PROCEDURE IF EXISTS AddElevColumns //
CREATE PROCEDURE AddElevColumns(IN addcol INT)
BEGIN

	IF addcol = 1 THEN
		ALTER TABLE data_format ADD COLUMN is_elev_data TINYINT DEFAULT 0;
	END IF;
	
	INSERT INTO data_format (column_name,description,is_elev_data) VALUES
     ('col','In elevation data list, this is a data frame of elevation data, in columnar format',1),
     ('x','In the columnar elevation data, this is x coordinate, matching gx of plot coordinate system in Tree and Stem tables, where called gx',1),
     ('y','In the columnar elevation data, this is x coordinate, matching gy of plot coordinate system in Tree and Stem tables',1),
     ('elev','Elevation in meters; in some plots, it is absolute elevation above-sea-level, but in other plots, relative elevation',1),
     ('mat',
       CONCAT('In elevation data list, this is a matrix of elevation data in format needed by map, image, contour programs. '
              'The columns of the matrix are columns in the plot (x dimension), and rows of the matrix are rows in the plot (y dimension). '
              'Most plots have elevation recorded over a grid of 5x5 m, so the dimensions of the matrix [dim(elev$mat)] are 1+ydim/5, 1+xdim/5, ',
              'with the 1 added because the elevation is recorded at all plot edges. ',
              'Counterintuitively, image and contour functions of R require this matrix be transposed for the map to be right, ',
              'ie, image(t(elev$mat)) works but image(elev$mat) is a faulty reflection of reality.'),
       1),
     ('xdim','The plot x dimension in meters, corresponding to plot and matrix columns',1),
     ('ydim','The plot y dimension in meters, corresponding to plot and matrix rows',1);
     
END //

DELIMITER ;

    
